<?php

namespace Ron\MVC;

/**
 * Umoznuje vyrendrovat obsah View nebo ViewObjectu do stringu
 */
class Renderer
{
    /** @var string|ViewObject  Jmeno php souboru, nebo ViewObject, nebo funkce, ktera neco renderuje */
    protected $view;

    /**
     * @param ViewObject|string|callable|null $view
     */
    public function __construct($view)
    {
        $this->view = $view;
    }


    /**
     * Spusti renderovani do bufferu - vse co se dale vyrenderuje se nerenderuje primo na vystup, ale pouze do bufferu,
     * ktery se pak vycte volanim metody stop()
     * @return Renderer
     */
    public function start()
    {
        ob_start();
        return $this;
    }

    /**
     * Ukonci renderovani
     * @return false|string
     */
    public function stop()
    {
        $result = ob_get_contents();
        ob_end_clean();

        if ($result === false) {
            throw new \Exception('Chyba v metode Renderer::stop');
        }

        return $result;
    }

    /**
     * @return string
     * @throws \Exception
     */
    public function renderContent()
    {
        $this ->start();

        if ($this ->view instanceof ViewObject) {
            $this ->view ->render();
        } else if (is_callable($this ->view)) {
            call_user_func($this ->view);
        } else if (is_string($this ->view)) {
            require $this ->view;
        }

        return $this ->stop();
    }


    /**
     * @param ViewObject|string|callable $view
     * @return string
     * @throws \Exception
     */
    public static function renderToString($view)
    {
        $renderer = new static($view);
        return $renderer ->renderContent();
    }
}
